==> membrane_core-0.6.0.tar: lib/membrane/core/parent/child_entry_parser.ex <==
defmodule Membrane.Core.Parent.ChildEntryParser do
  @moduledoc false

  alias Membrane.{ChildEntry, ParentError, ParentSpec}

  @type raw_child_entry_t :: %ChildEntry{
          name: Membrane.Child.name_t(),
          module: module,
          options: struct | nil,
          component_type: :element | :bin,
          pid: pid | nil,
          clock: Membrane.Clock.t() | nil,
          sync: Membrane.Sync.t() | nil,
          pending?: boolean()
        }

  @spec from_spec(ParentSpec.children_spec_t() | any) :: [raw_child_entry_t] | no_return
  def from_spec(children_spec) when is_map(children_spec) or is_list(children_spec) do
    children_spec |> Enum.map(&parse_child/1)
  end

  defp parse_child({name, %module{} = options}) do
    %ChildEntry{
      name: name,
      module: module,
      options: options,
      component_type: component_type(module),
      pending?: false
    }
  end

  defp parse_child({name, module}) when is_atom(module) do
    options = module |> Bunch.Module.struct()

    %ChildEntry{
      name: name,
      module: module,
      options: options,
      component_type: component_type(module),
      pending?: false
    }
  end

  defp parse_child(config) do
    raise ParentError, "Invalid children config: #{inspect(config, pretty: true)}"
  end

  defp component_type(module) do
    cond do
      Membrane.Element.element?(module) ->
        :element

      Membrane.Bin.bin?(module) ->
        :bin

      true ->
        raise ParentError, """
        Child module #{inspect(module)} is neither Membrane Element nor Bin.
        Make sure that given module is the right one, implements proper behaviour
        and all needed dependencies are properly specified in the Mixfile.
        """
    end
  end
end

==> membrane_core-0.6.1.tar: lib/membrane/core/parent/child_entry_parser.ex <==
defmodule Membrane.Core.Parent.ChildEntryParser do
  @moduledoc false

  alias Membrane.{ChildEntry, ParentError, ParentSpec}

  @type raw_child_entry_t :: %ChildEntry{
          name: Membrane.Child.name_t(),
          module: module,
          options: struct | nil,
          component_type: :element | :bin
        }

  @spec from_spec(ParentSpec.children_spec_t() | any) :: [raw_child_entry_t] | no_return
  def from_spec(children_spec) when is_map(children_spec) or is_list(children_spec) do
    children_spec |> Enum.map(&parse_child/1)
  end

  defp parse_child({name, %module{} = options}) do
    %ChildEntry{
      name: name,
      module: module,
      options: options,
      component_type: component_type(module)
    }
  end

  defp parse_child({name, module}) when is_atom(module) do
    options = module |> Bunch.Module.struct()

    %ChildEntry{
      name: name,
      module: module,
      options: options,
      component_type: component_type(module)
    }
  end

  defp parse_child(config) do
    raise ParentError, "Invalid children config: #{inspect(config, pretty: true)}"
  end

  defp component_type(module) do
    cond do
      Membrane.Element.element?(module) ->
        :element

      Membrane.Bin.bin?(module) ->
        :bin

      true ->
        raise ParentError, """
        Child module #{inspect(module)} is neither Membrane Element nor Bin.
        Make sure that given module is the right one, implements proper behaviour
        and all needed dependencies are properly specified in the Mixfile.
        """
    end
  end
end
